/*
 * gas.c
 *
 * GKrellM Another Sensor plugin
 * Copyright (c) 2002 Masaharu FUJITA
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * You may contact the author by:
 *      e-mail: m@fjts.org
 *      url: http://fjts.org/~m/Soft/GAS/
 */

#include <gkrellm2/gkrellm.h>
#include <sys/sysctl.h>

#define VERSION "2.0.2"

#define PLUGIN "GAS"
#define PLUGIN_KEYWORD "gas"
#define TZ_ZEROC 2732
#define TZ_KELVTOC(x) ((gfloat)((x) - TZ_ZEROC) / 10.0)
#define Celsius2Fahrenheit(x) ((gfloat)((x) * 1.8) + 32.0)
#define Fahrenheit2Celsius(x) ((gfloat)((x) - 32.0) / 18.0)

#define ViewTypeNum 2
typedef enum {
  Celsius,
  Fahrenheit,
} ViewType;

#define KRELL_MAX 100
#define SCALE_MAX 500

static GkrellmMonitor *monitor;

static gint chart_style_id;
static GkrellmChart *chart = NULL;
static GkrellmChartconfig *chart_config;
gint enable_chart = TRUE;
static GtkWidget *chart_button = NULL;

gint meter_style_id;
GkrellmPanel *panel;
GkrellmKrell *krell;
GkrellmDecal *decal;
char *label;
gint enable_slider = TRUE;
static GtkWidget *slider_button = NULL;

gdouble slider_start = 0;
gdouble slider_end = 80;
static GtkWidget *slider_start_entry = NULL;
static GtkWidget *slider_end_entry = NULL;

gint view_type = Celsius;
GtkWidget *config_radio[ViewTypeNum+1];

static gdouble total = 0;
int gas_oid[CTL_MAXNAME + 2];
size_t gas_oidlen;

static gint get_sensor()
{
  int value;
  size_t size = sizeof(value);
  
  if ( sysctl(gas_oid, gas_oidlen, &value, &size, NULL, 0) < 0 )
    return FALSE;

  total = TZ_KELVTOC(value);

  return TRUE;
}

static gint check_sensor()
{
  int oid[CTL_MAXNAME + 2];
  size_t oidlen = sizeof(oid);
  static int oid_name2oid[2] = { 0, 3 };
  static char *mib_temp = "hw.acpi.thermal.tz0.temperature";
  
  if ( sysctl(oid_name2oid, 2, oid, &oidlen,
	      (void *)mib_temp, strlen(mib_temp)) < 0 )
    return FALSE;
  
  memcpy(gas_oid, oid, oidlen);
  gas_oidlen = oidlen / sizeof(int);
  
  return TRUE;
}

static void set_label()
{
  switch (view_type) {
  case Celsius:
    label = g_strdup_printf("%.1fC", total);
    break;
  case Fahrenheit:
    label = g_strdup_printf("%.1fF", Celsius2Fahrenheit(total));
    break;
  default:
    label = strdup("");
  }
}

static void draw_chart_extra(GkrellmChart *chart)
{
  if (!chart)
    return;

  set_label();  
  gkrellm_draw_chart_text(chart, chart_style_id, label);
}

static void draw_chart(GkrellmChart *cp)
{
  gkrellm_draw_chartdata(cp);
  if (view_type == Celsius || view_type == Fahrenheit)
    draw_chart_extra(cp);
  gkrellm_draw_chart_to_screen(cp);
}

static gint expose_event(GtkWidget *widget, GdkEventExpose *ev)
{
  GdkPixmap *pixmap = NULL;
  
  if ( widget == chart->drawing_area ) {
    pixmap = chart->pixmap;
    gdk_draw_pixmap(widget->window,
	  	    widget->style->fg_gc[GTK_WIDGET_STATE (widget)], pixmap,
		    ev->area.x, ev->area.y, ev->area.x, ev->area.y,
		    ev->area.width, ev->area.height);
  } else if ( widget == chart->panel->drawing_area ) {
    pixmap = chart->panel->pixmap;
    gdk_draw_pixmap(widget->window,
	  	    widget->style->fg_gc[GTK_WIDGET_STATE (widget)], pixmap,
		    ev->area.x, ev->area.y, ev->area.x, ev->area.y,
		    ev->area.width, ev->area.height);
  } else if (widget == panel->drawing_area) {
    gdk_draw_pixmap(widget->window,
                    widget->style->fg_gc[GTK_WIDGET_STATE(widget)],
                    panel->pixmap, ev->area.x, ev->area.y,
                    ev->area.x, ev->area.y, ev->area.width, ev->area.height);
  }
  
  return FALSE;
}

static gint clicked_event(GtkWidget *widget, GdkEventButton *event)
{
  if (!enable_chart)
    return TRUE;

  switch (event->button) {
  case 1:
    if (view_type == Celsius || view_type == Fahrenheit)
      view_type = 2;
    else
      view_type = Celsius;
    gkrellm_config_modified();
    break;
  case 2:
    if (view_type == Celsius)
      view_type = Fahrenheit;
    else if (view_type == Fahrenheit)
      view_type = Celsius;
    gkrellm_config_modified();
    break;
  case 3:
    gkrellm_chartconfig_window_create(chart);
    break;
  }
  
  return TRUE;
}

static void create_plugin(GtkWidget *vbox, gint first)
{
  GkrellmStyle *style;
  GkrellmKrell *k;
  GkrellmChartdata *cd;
  GkrellmPiximage *krell_image;
  GkrellmStyle *panel_style;
  GkrellmStyle *slider_style;

  panel_style = gkrellm_meter_style(meter_style_id);
  
  if (first) {
    chart = gkrellm_chart_new0();
    chart->panel = gkrellm_panel_new0();
    panel = gkrellm_panel_new0();
  }

  /* Chart */
  style = gkrellm_panel_style(chart_style_id);

  gkrellm_chart_create(vbox, monitor, chart, &chart_config);
  cd = gkrellm_add_default_chartdata(chart, "Sensor");
  gkrellm_monotonic_chartdata(cd, FALSE);
  gkrellm_set_chartconfig_grid_resolution(chart_config,
                                           SCALE_MAX / FULL_SCALE_GRIDS);
  gkrellm_set_chartconfig_auto_grid_resolution(chart_config, FALSE);
  gkrellm_alloc_chartdata(chart);
  gkrellm_set_draw_chart_function(chart, draw_chart, chart);

  k = gkrellm_create_krell(chart->panel,
			   gkrellm_krell_meter_piximage(chart_style_id), style);

  gkrellm_monotonic_krell_values(k, FALSE);
  gkrellm_set_krell_full_scale(k, KRELL_MAX, 1);
  gkrellm_panel_configure(chart->panel, g_strdup("Sensor"), style);
  gkrellm_panel_create(vbox, monitor, chart->panel);

  if (!enable_chart) {
    gkrellm_monitor_height_adjust(-chart->panel->h);
    gkrellm_monitor_height_adjust(-chart->h);
    gtk_widget_hide(chart->panel->hbox);
    gtk_widget_hide(chart->box);
  }

  /* Slider */
  set_label();
  panel->textstyle = gkrellm_meter_textstyle(meter_style_id);

  decal = gkrellm_create_decal_text(panel, "M8", panel->textstyle,
                                    gkrellm_meter_style(meter_style_id),
                                    -1, -1, -1 );

  gkrellm_draw_decal_text(panel, decal, label, -1 );
  gkrellm_panel_configure(panel, NULL, panel_style);
  gkrellm_panel_create(vbox,monitor,panel);

  if (!enable_slider) {
    gkrellm_monitor_height_adjust(-panel->h);
    gtk_widget_hide(panel->hbox);
  }

  slider_style = gkrellm_krell_slider_style();
  krell_image = gkrellm_krell_slider_piximage();
  krell = gkrellm_create_krell(panel,krell_image,slider_style);
  krell->y0 = (panel->h - krell->h_frame)/2;
  krell->full_scale = slider_end;

  if ( first ) {
    gtk_signal_connect(GTK_OBJECT(chart->drawing_area), "expose_event",
		       GTK_SIGNAL_FUNC(expose_event), NULL);
    gtk_signal_connect(GTK_OBJECT(chart->panel->drawing_area), "expose_event",
		       GTK_SIGNAL_FUNC(expose_event), NULL);
    gtk_signal_connect(GTK_OBJECT(panel->drawing_area), "expose_event",
		       GTK_SIGNAL_FUNC(expose_event), NULL);
    gtk_signal_connect(GTK_OBJECT(chart->drawing_area), "button_press_event",
		       GTK_SIGNAL_FUNC(clicked_event), NULL);
  }

  gkrellm_update_krell(panel,krell,(gulong)total);
  gkrellm_draw_panel_layers(panel);
}

static void update_plugin()
{
  GkrellmKrell *chart_krell;

  if (GK.five_second_tick) {
    get_sensor();
    set_label();

    /* Slider */
    krell->previous = 0;
    gkrellm_update_krell(panel,krell,(gulong)total-slider_start);
    gkrellm_draw_decal_text(panel, decal, label, -1);
    gkrellm_draw_panel_layers(panel);
  }

  if (GK.second_tick) {
    gkrellm_store_chartdata(chart, 0, (gulong)total, 0);
    draw_chart(chart);
  }
  
  chart_krell = KRELL(chart->panel);
  gkrellm_update_krell(chart->panel, chart_krell,(gulong)total);
  gkrellm_draw_panel_layers(chart->panel);
}

static void create_tab(GtkWidget *tab)
{
  GtkWidget *tabs;
  GtkWidget *vbox, *about, *label, *hbox, *frame, *tmpbox;
  GSList *view_group = NULL;
  gchar *tmp;
  gint i;

  gchar *about_text = 
    g_strdup_printf("%s %s\n" \
		    "GKrellM GKrellM Another Sensor Plugin \n" \
		    "\n" \
		    "Copyright (C) 2002 Masaharu FUJITA\n" \
		    "m@fjts.org\n" \
		    "http://fjts.org/~m/Soft/GAS/\n" \
		    "\n" \
		    "Released under the GNU General Public License\n",
		    PLUGIN, VERSION);

  gchar *view_name[] = { "Show Celsius", "Show Fahrenheit",  "None" };
  
  tabs = gtk_notebook_new();
  gtk_notebook_set_tab_pos(GTK_NOTEBOOK(tabs), GTK_POS_TOP);
  gtk_box_pack_start(GTK_BOX(tab), tabs, TRUE, TRUE, 0);

  vbox = gkrellm_gtk_framed_notebook_page(tabs, "Setup");
  gkrellm_gtk_check_button(vbox, &chart_button, enable_chart, TRUE, 0,
		       "Enable chart");
  gkrellm_gtk_check_button(vbox, &slider_button, enable_slider, TRUE, 0,
		       "Enable slider");

  /* View Type */
  vbox = gkrellm_gtk_framed_notebook_page(tabs, "View Type");

  for ( i = 0 ; i <= ViewTypeNum ; i++ ) {
    config_radio[i] = gtk_radio_button_new_with_label(view_group,
                                                      view_name[i]);
    view_group = gtk_radio_button_group(GTK_RADIO_BUTTON(config_radio[i]));

    if ( i == view_type )
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config_radio[i]), TRUE);

    gtk_widget_show(config_radio[i]);
    gtk_box_pack_start(GTK_BOX(vbox), config_radio[i], TRUE, TRUE, 0);
  }

  /* Slider Option */
  vbox = gkrellm_gtk_framed_notebook_page(tabs, "Slider Option");

  frame = gtk_frame_new("Point Values Setting");
  tmpbox = gtk_vbox_new(FALSE,0);
  gtk_container_add(GTK_CONTAINER (frame), tmpbox);

  hbox = gtk_hbox_new(FALSE,0);
  label = gtk_label_new(" Start Point Value: ");
  gtk_box_pack_start(GTK_BOX (hbox), label, FALSE, FALSE, 0);
  slider_start_entry = gtk_entry_new_with_max_length(3);
  gtk_widget_set_usize(slider_start_entry, 31, 0);
  tmp = g_strdup_printf("%g", slider_start);
  gtk_entry_set_text(GTK_ENTRY(slider_start_entry), tmp);
  gtk_box_pack_start(GTK_BOX (hbox), slider_start_entry, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX (tmpbox), hbox, TRUE, TRUE, 0);

  hbox = gtk_hbox_new(FALSE,0);
  label = gtk_label_new(" End  Point Value: ");
  gtk_box_pack_start(GTK_BOX (hbox), label, FALSE, FALSE, 0);
  slider_end_entry = gtk_entry_new_with_max_length(3);
  gtk_widget_set_usize(slider_end_entry, 31, 0);
  tmp = g_strdup_printf("%g", slider_end);
  gtk_entry_set_text(GTK_ENTRY(slider_end_entry), tmp);
  gtk_box_pack_start(GTK_BOX (hbox), slider_end_entry, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX (tmpbox), hbox, TRUE, TRUE, 0);

  gtk_box_pack_start(GTK_BOX (vbox), frame, TRUE, TRUE, 0);

  label =
    gtk_label_new("Currently, unit of setting point values is Celsius only.");
  gtk_box_pack_start(GTK_BOX (vbox), label, TRUE, TRUE, 0);

  /* About */
  vbox = gkrellm_gtk_framed_notebook_page(tabs, "About");
  about = gtk_label_new(about_text);
  gtk_box_pack_start(GTK_BOX (vbox), about, TRUE, TRUE, 0);
}

static void apply_config()
{
  gint i;
  gint flag;
  gdouble old_slider_end;

  flag = enable_chart;
  enable_chart = GTK_TOGGLE_BUTTON(chart_button)->active;
  if (flag != enable_chart) {
    if (enable_chart) {
      gkrellm_monitor_height_adjust(chart->panel->h);
      gkrellm_monitor_height_adjust(chart->h);
      gtk_widget_show(chart->panel->hbox);
      gtk_widget_show(chart->box);
    } else {
      gkrellm_monitor_height_adjust(-chart->panel->h);
      gkrellm_monitor_height_adjust(-chart->h);
      gtk_widget_hide(chart->panel->hbox);
      gtk_widget_hide(chart->box);
    }
  }

  flag = enable_slider;
  enable_slider = GTK_TOGGLE_BUTTON(slider_button)->active;
  if (flag != enable_slider) {
    if (enable_slider) {
      gkrellm_monitor_height_adjust(panel->h);
      gtk_widget_show(panel->hbox);
    } else {
      gkrellm_monitor_height_adjust(-panel->h);
      gtk_widget_hide(panel->hbox);
    }
  }

  for ( i = 0 ; i <= ViewTypeNum ; i++ ) {
    if ( GTK_TOGGLE_BUTTON(config_radio[i])->active ) {
      view_type = i;
      break;
    }
  }

  old_slider_end = slider_end;
  slider_end = g_strtod(gtk_entry_get_text(GTK_ENTRY (slider_end_entry)),NULL);
  slider_start = g_strtod(gtk_entry_get_text(GTK_ENTRY (slider_start_entry)),NULL);
  if ( slider_end != old_slider_end )
    krell->full_scale = slider_end;

  set_label();
  gkrellm_config_modified();
}

static void save_config(FILE *f)
{
  fprintf(f, "%s enable_chart %d\n", PLUGIN_KEYWORD, enable_chart);
  fprintf(f, "%s enable_slider %d\n", PLUGIN_KEYWORD, enable_slider);
  fprintf(f, "%s view_type %d\n", PLUGIN_KEYWORD, view_type);
  fprintf(f, "%s slider_start %g\n", PLUGIN_KEYWORD, slider_start);
  fprintf(f, "%s slider_end %g\n", PLUGIN_KEYWORD, slider_end);

  gkrellm_save_chartconfig(f, chart_config, PLUGIN_KEYWORD, NULL);
}

static void load_config(gchar *arg)
{
  gchar config[32], item[CFG_BUFSIZE];
  
  if ( sscanf(arg, "%s %[^\n]", config, item) != 2 )
    return;
  
  if ( strcmp(config, "enable_chart") == 0 )
    sscanf(item, "%d", &enable_chart);
  else   if ( strcmp(config, "enable_slider") == 0 )
    sscanf(item, "%d", &enable_slider);
  else if ( strcmp(config, "view_type") == 0 )
    sscanf(item, "%d", &view_type);
  else if ( strcmp(config, "slider_start") == 0 )
    sscanf(item, "%lf", &slider_start);
  else if ( strcmp(config, "slider_end") == 0 )
    sscanf(item, "%lf", &slider_end);
  else if (!strcmp(config, GKRELLM_CHARTCONFIG_KEYWORD))
    gkrellm_load_chartconfig (&chart_config, item, 1);
}

static GkrellmMonitor gas  =
{
        PLUGIN,                 /* Name, for config tab. */
        0,                      /* Id, 0 if a plugin */
        create_plugin,          /* The create_plugin() function */
        update_plugin,          /* The update_plugin() function */

        create_tab,             /* Create_plugin_tab() config func*/
        apply_config,           /* The apply_plugin_config() function*/

        save_config,            /* The save_plugin_config() function */
        load_config,            /* The load_plugin_config() function */
        PLUGIN_KEYWORD,         /* Config Keyword */

        NULL,                   /* Undefined 2 */
        NULL,                   /* Undefined 1 */
        NULL,                   /* Undefined 0 */

        MON_APM,                /* Insert plugin before this monitor */
        NULL,                   /* Handle if a plugin, filled by GKrellM */
        NULL                    /* Path if a plugin, filled by GKrellM */
};

GkrellmMonitor *gkrellm_init_plugin()
{
  
  if (!check_sensor())
    return NULL;
  get_sensor();
  set_label();

  monitor = &gas;
  chart_style_id = gkrellm_add_chart_style(&gas, PLUGIN_KEYWORD);
  meter_style_id = gkrellm_add_meter_style(&gas, PLUGIN_KEYWORD);

  return &gas;
}
